# written by Anirban Bhattacharya
# do not share without permission


# Rue's sampler for MVN
# sample from N(Q^{-1}b, Q^{-1}) 
mvnrue = function(Q,b){ 
  d = length(b)
  L = t(chol(Q))                      # Q = LL'; R returns right Cholesky factor
  yr = solve(t(L),rnorm(d))           # z~N(0,I_d), L'y=z
  vr = solve(L,b)
  thetar = solve(t(L),vr)             # L'theta = v, where Lv = b
  samp = yr + thetar
  return(samp)
}


# -- function to fit probit regression model with ridge prior -- #
# Pr(y_i=1|beta) = Phi(x_i'beta)
# beta ~ N(0,xi^{-1}I) for a fixed xi
# Use Albert-Chib + Holmes-Held data augmentation

probit_ridge_HH = function(y,X,xi,BURNIN=1000,MCMC=4000,THIN=1){
  # y is nx1 binary vector, X is nxp design matrix
  # xi is the prior precision for beta
  
  library(tmg)
  sgn = 2*y-1             # sign of the y's (plus or minus if 1 or 0 resp.)
  rr = rep(0,n)           # to call rtmg
  ff = diag(sgn)          # to call rtmg
  gg = rep(0,n)           # to call rtmg
  init = runif(n)*sgn     # to call rtmg
  
  # global constants
  n = nrow(X)
  d = ncol(X)
  NITER = BURNIN + MCMC
  EFF = MCMC/THIN
  I_n = diag(n)
  I_d = diag(d)
  H = t(X)%*%X
  G = X%*%t(X)                       # G = XX'
  Q = solve(I_n + (1/xi)*G)          # to call rtmg
  
  # initialize z and beta
  z = init
  beta = rnorm(d)
  
  # output files  
  betaout = matrix(0,nrow=EFF,ncol=d)  
  zout = matrix(0,nrow=EFF,ncol=n)
  
  # begin MCMC
  for(g in 1:NITER) {
    # -- update z -- #
    z = rtmg(1,Q,rr,init,ff,gg,q=NULL,burn.in=0)
    z = as.vector(z)
    init = z
    
    # -- update beta -- #
    bb = t(X)%*%z
    beta = mvnrue((H+xi*I_d),bb)
    
    # -- save stuff -- #
    if(g > BURNIN && g%%THIN==0) {
      betaout[(g-BURNIN)/THIN,] = beta
      zout[(g-BURNIN)/THIN,] = z
    }
    #if(g%%10==0){print(g)}
  }
  
  postmean = apply(betaout,2,mean)
  return(list('Betaout' = betaout,'postmean_beta' = postmean,'Zout' = zout)) 
  
}